import matplotlib.pyplot as plt
import numpy as np
import matplotlib.patches as mpatches
import pandas as pd
import numpy as np
from scipy.stats import chi2_contingency


def bar_graph():
    background_colors = ['#FFDDC1', '#C1D1FF']
    category_labels = ['Correct Style', 'Incoreect Style']
    category_ranges = [(0, 7),(7, 12)]

    #correct_cate = ['5G', 'Supplements', 'Vaccines', 'School education', 'Climate change','School education', '5G']
    #neutral_values = [0.44, 0.70, 0.65, 1.07, 1.68, 0.58, 1.91]  
    #styled_values = [0.76, 0.67, 0.98, 6.07, 3.31, 1.07, 2.99] 

    #incorrect_cate = ['5G','School education','Vaccines','5G', 'School education']
    #neutral2 = [0.46, 1.32, 1.27, 1.88, 0.76]
    #style2 = [0.57, 4.58, 1.61, 4.86, 2.10]
    # Extract values from the csv file
    df = pd.read_csv('Facebook_results.csv')
    df = df[pd.to_numeric(df['Pairs'], errors='coerce').notnull()]
    df['Pairs'] = df['Pairs'].astype(int)
    df = df[['Pairs', 'Topic', 'Style', 'Target', 'click through rate']]
    df['click through rate'] = df['click through rate'].astype(float)
    correct_cate = []
    neutral_values = []
    styled_values = []
    incorrect_cate = []
    neutral2 = []
    style2 = []
    for pair in df['Pairs'].unique():
        pair_rows = df[df['Pairs'] == pair]
        neutral_row = pair_rows[pair_rows['Style'] == 'Neutral'].iloc[0]
        style_row = pair_rows[pair_rows['Style'] != 'Neutral'].iloc[0]
        if style_row['Style'] == style_row['Target']:
            styled_values.append(style_row['click through rate'])
            correct_cate.append(style_row['Topic'])
            neutral_values.append(neutral_row['click through rate'])
        else:
            incorrect_cate.append(style_row['Topic'])
            style2.append(style_row['click through rate'])
            neutral2.append(neutral_row['click through rate'])
   
    plt.figure(figsize=(16, 8))
    x = np.arange(len(correct_cate)+len(incorrect_cate))
    bar_width = 0.4
    bar_texts = ['***', ' ', '*', '***','***','***', '***', ' ', '***', ' ','***', '***']
    bars1 = plt.bar(x - bar_width/2, neutral_values + neutral2, bar_width, color='lightsteelblue', label='Neutral')
    bars2 = plt.bar(x + bar_width/2, styled_values + style2, bar_width, color='royalblue', label='Styled')
    
    plt.xlabel('Topics',fontsize=15)
    plt.ylabel('CTR',fontsize=15)
    
    ax = plt.gca()
    ax.spines['top'].set_visible(False)
    ax.spines['right'].set_visible(False)
    for (start, end), color in zip(category_ranges, background_colors):
        ax.axvspan(start - 0.5, end - 0.5, color=color, alpha=0.2)
    category_patches = [mpatches.Patch(color=color, label=label) for color, label in zip(background_colors, category_labels)]
    plt.legend(handles=[mpatches.Patch(color='lightsteelblue', label='Neutral'),
                        mpatches.Patch(color='royalblue', label='Styled')] + category_patches)
 
    plt.ylim(top=7)
    plt.xticks(x, correct_cate+incorrect_cate,rotation=35,ha='right')
    for idx, bar in enumerate(bars2):
        text = bar_texts[idx]
        plt.text(bar.get_x() + bar.get_width() / 2, bar.get_height(), text,
                 ha='center', va='bottom', color='black')
    labels = [label for label in plt.gca().get_xticklabels()]
    print(labels)
    for i, label in enumerate(labels):
        if i in [0,1,2,3,4,7,8,9]: 
            label.set_weight('bold') 

    first_legend = plt.legend(handles=[bars1, bars2], loc='upper left', fontsize=14)
    ax.add_artist(first_legend)  # This adds the first legend back after the second is created

    category_patches = [mpatches.Patch(color=color, label=label) for color, label in zip(background_colors, category_labels)]
    plt.savefig("effect_personalise_style.pdf", bbox_inches="tight")
    plt.show()



def chi_squared_tests():
    df = pd.read_csv('Facebook_results.csv')
    df = df[pd.to_numeric(df['Pairs'], errors='coerce').notnull()]
    df['Pairs'] = df['Pairs'].astype(int)
    df = df[['Pairs', 'Topic', 'Style', 'Target', 'Impressions', 'Results']]
    df['Impressions'] = df['Impressions'].astype(int)
    df['Results'] = df['Results'].astype(int)
    
    for pair in df['Pairs'].unique():
        pair_rows = df[df['Pairs'] == pair]
        neutral_row = pair_rows[pair_rows['Style'] == 'Neutral'].iloc[0]
        style_row = pair_rows[pair_rows['Style'] != 'Neutral'].iloc[0]
        clicks_A = neutral_row['Results']#216
        non_clicks_A = neutral_row['Impressions']-clicks_A#11337-216
        clicks_B = style_row['Results']#328
        non_clicks_B = style_row['Impressions']-clicks_B#10954-328
        print(style_row['Topic'], style_row['Target'])
        chi_squared_test(clicks_A, non_clicks_A, clicks_B, non_clicks_B)


def chi_squared_test(clicks_A, non_clicks_A, clicks_B, non_clicks_B):
    observed = np.array([[clicks_A, non_clicks_A], [clicks_B, non_clicks_B]])
    chi2, p, dof, expected = chi2_contingency(observed)

    print("Chi-square:", chi2)
    print("P-value:", p)
    print("Degrees of freedom:", dof)
    print("Expected frequencies:", expected)
    # correction: we have 12 comparison
    alpha_1 = 0.05/12
    alpha_2 = 0.01/12
    alpha_3 = 0.001/12

    if p < alpha_3:
        print("Reject null hypothesis: There is a significant difference.***")
    elif p < alpha_2:
        print("Reject null hypothesis: There is a significant difference.**")
    elif p < alpha_1:
        print("Reject null hypothesis: There is a significant difference.*")
    else:
        print("Fail to reject null hypothesis: No significant difference.")


chi_squared_tests()
#bar_graph()
